package com.cweb.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.cweb.config.SysConfig;
import com.cweb.config.SysConst;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.security.SessionObject;
import com.hai.common.security.UserCenter;
import com.hai.common.utils.CoordCommonUtil;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.config.CommonSysConst;
import com.hai.config.QianZhuConfig;
import com.hai.config.ThirdProductConfig;
import com.hai.entity.*;
import com.hai.enum_type.ProductImgEnum;
import com.hai.model.ApiStarbucksProductsModel;
import com.hai.model.BsProductConfigModel;
import com.hai.model.HighUserModel;
import com.hai.model.ResponseData;
import com.hai.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.w3c.dom.ls.LSException;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;


@Controller
@RequestMapping(value = "/highThirdParty")
@Api(value = "订单接口")
public class HighThirdPartyController {

    private static Logger log = LoggerFactory.getLogger(HighThirdPartyController.class);

    @Resource
    private ApiProductService apiProductService;

    @Resource
    private ThirdProductConfig thirdProductConfig;

    @Resource
    private HighOrderService highOrderService;

    @Resource
    private BsCompanyService bsCompanyService;

    @Resource
    private CommonService commonService;

    @Autowired
    private UserCenter userCenter;

    @Resource
    private BsConfigService bsConfigService;


    @RequestMapping(value = "/getAllCity", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询所有城市")
    public ResponseData getAllCity(@RequestParam(name = "cityName", required = false) String cityName) {
        try {
            return ResponseMsgUtil.success(apiProductService.getAllCity(cityName));

        } catch (Exception e) {
            log.error("HighUserCardController --> oilCardRefund() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/getStarbucksProducts", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询星巴克商品")
    public ResponseData getStarbucksProducts(@RequestParam(name = "name", required = false) String name,
                                             @RequestParam(value = "regionId", required = true) String regionId) {
        try {

            SecRegion region = commonService.getParentByRegion(Long.valueOf(regionId));
            BigDecimal productDiscount = null;
            if (region != null) {
                BsCompany bsCompany = bsCompanyService.selectCompanyByRegion(region.getRegionId().toString());
                productDiscount = thirdProductConfig.getProductConfigNum(1, bsCompany.getId());
            }

            if (productDiscount == null) {
                log.error("getRechargeProduct error!", "查询失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "当前地区未配置产品");
            }

            List<String> CategoryNameList = apiProductService.getCategoryName();

            List<ApiStarbucksProducts> starbucksProductsList = new ArrayList<>();
            int i = 9999;
            for (String s : CategoryNameList) {
                i++;
                ApiStarbucksProducts starbucksProducts = new ApiStarbucksProducts();
                Map<String, Object> map = new HashMap<>();
                map.put("name", name);
                map.put("categoryName", s);
                List<ApiStarbucksProducts> list = apiProductService.getStarbucksProducts(map);
                for (ApiStarbucksProducts apiStarbucksProducts : list) {
                    apiStarbucksProducts.setExt1(String.valueOf(i));
                    apiStarbucksProducts.setSalesVentiPrice(apiStarbucksProducts.getSalesVentiPrice().add(apiStarbucksProducts.getSalesVentiPrice().multiply(productDiscount.divide(new BigDecimal(100)))).setScale(2, RoundingMode.HALF_UP));
                    apiStarbucksProducts.setSalesTallPrice(apiStarbucksProducts.getSalesTallPrice().add(apiStarbucksProducts.getSalesTallPrice().multiply(productDiscount.divide(new BigDecimal(100)))).setScale(2, RoundingMode.HALF_UP));
                    apiStarbucksProducts.setSalesGrandePrice(apiStarbucksProducts.getSalesGrandePrice().add(apiStarbucksProducts.getSalesGrandePrice().multiply(productDiscount.divide(new BigDecimal(100)))).setScale(2, RoundingMode.HALF_UP));
                    starbucksProductsList.add(apiStarbucksProducts);
                }
                starbucksProducts.setName(s);
                starbucksProducts.setExt2(SysConst.getSysConfig().getImgUrl() + ProductImgEnum.getNameByImgUrl(s));
                starbucksProducts.setId((long) i);
                starbucksProductsList.add(starbucksProducts);
            }

            return ResponseMsgUtil.success(starbucksProductsList);

        } catch (Exception e) {
            log.error("HighUserCardController --> oilCardRefund() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/getStarbucksStore", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询星巴克门店")
    public ResponseData getStarbucksStore(@RequestParam(name = "lat", required = true) String lat,
                                          @RequestParam(name = "lon", required = true) String lon) {
        try {
            JSONObject jsonObject = QianZhuConfig.getStarbucksStore(lat, lon);
            if (jsonObject.getBoolean("success")) {
                return ResponseMsgUtil.success(jsonObject.getJSONArray("data"));
            } else {
                log.error("getRechargeProduct error!", "查询失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, jsonObject.getString("message"));
            }

        } catch (Exception e) {
            log.error("HighUserCardController --> oilCardRefund() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


    @RequestMapping(value = "/getKfcStoresList", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询肯德基门店")
    public ResponseData getKfcStoresList(@RequestParam(name = "storeName", required = false) String storeName,
                                         @RequestParam(name = "cityCode", required = true) String cityCode,
                                         @RequestParam(name = "lat", required = true) String lat,
                                         @RequestParam(name = "lon", required = true) String lon) {
        try {

            Map<String, Object> map = new HashMap<>();
            map.put("storeName", storeName);
            map.put("cityCode", cityCode);

            List<ApiKfcStores> list = apiProductService.getKfcStoresList(map);

            if (StringUtils.isNotBlank(lat)) {
                for (ApiKfcStores stores : list) {
                    double distance = CoordCommonUtil.getDistance(Double.valueOf(stores.getLat()), Double.valueOf(stores.getLon()), Double.valueOf(lat), Double.valueOf(lon));
                    stores.setDistance(String.valueOf(Math.round(distance / 100d) / 10d));
                }
            }

            return ResponseMsgUtil.success(list.stream().sorted(Comparator.comparing(ApiKfcStores::getDistance)).collect(Collectors.toList()));

        } catch (Exception e) {
            log.error("HighUserCardController --> oilCardRefund() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/listByStoreCode", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询肯德基门店菜单")
    public ResponseData listByStoreCode(@RequestParam(name = "storeCode", required = false) String storeCode, @RequestParam(value = "regionId", required = true) String regionId) {
        try {

            JSONObject jsonObject = QianZhuConfig.listByStoreCode(storeCode);

            JSONArray array = jsonObject.getJSONArray("data");

            if (array == null) {
                log.error("addOrder error!");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未查询到任何门店");
            }

            BigDecimal productDiscount = null;
            SecRegion region = commonService.getParentByRegion(Long.valueOf(regionId));
            if (region != null) {
                BsCompany bsCompany = bsCompanyService.selectCompanyByRegion(region.getRegionId().toString());
                productDiscount = thirdProductConfig.getProductConfigNum(1, bsCompany.getId());
            }

            if (productDiscount == null) {
                log.error("getRechargeProduct error!", "查询失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "当前地区未配置产品");
            }
            int i = 0;

            for (Object data : array) {
                JSONObject dataObject = (JSONObject) data;
                i++;
                dataObject.put("kfcId", i);
                if (dataObject.getJSONArray("products") != null) {
                    for (Object products : dataObject.getJSONArray("products")) {
                        JSONObject productsObject = (JSONObject) products;
                        BigDecimal productsPrice = productsObject.getBigDecimal("price");
                        productsObject.put("salePrice", productsPrice.add(productsPrice.multiply(productDiscount).divide(new BigDecimal(100))).setScale(2, RoundingMode.HALF_UP));

                        if (productsObject.getJSONArray("specGroups") != null) {
                            for (Object specGroups : productsObject.getJSONArray("specGroups")) {
                                JSONObject specGroupsObject = (JSONObject) specGroups;
                                BigDecimal specGroupsPrice = specGroupsObject.getBigDecimal("price");
                                specGroupsObject.put("salePrice", specGroupsPrice.add(specGroupsPrice.multiply(productDiscount).divide(new BigDecimal(100))).setScale(2, RoundingMode.HALF_UP));
                            }
                        }
                    }
                }
            }

            if (jsonObject.getBoolean("success")) {
                return ResponseMsgUtil.success(array);
            } else {
                log.error("getRechargeProduct error!", "查询失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, jsonObject.getString("message"));
            }

        } catch (Exception e) {
            log.error("HighUserCardController --> oilCardRefund() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/getThirdPartyByDetail", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询第三方产品详情")
    public ResponseData getThirdPartyByDetail(@RequestParam(name = "platformId", required = true) Integer platformId,
                                              @RequestParam(name = "productType", required = true) Integer productType,
                                              @RequestParam(name = "regionId", required = true) String regionId) {
        try {

            SecRegion region = commonService.getParentByRegion(Long.valueOf(regionId));

            if (region == null) {
                log.error("getRechargeProduct error!", "查询失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, regionId + "当前地区未配置产品");
            }

            BsCompany bsCompany = bsCompanyService.selectCompanyByRegion(region.getRegionId().toString());

            return ResponseMsgUtil.success(thirdProductConfig.getThirdPartyByDetail(platformId , productType , bsCompany.getId()));

        } catch (Exception e) {
            log.error("HighUserCardController --> oilCardRefund() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


    @RequestMapping(value = "/addOrder", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "增加订单")
    public ResponseData addOrder(@RequestBody JSONObject object, HttpServletRequest request) {
        try {

            // 用户
            SessionObject sessionObject = userCenter.getSessionObject(request);
            HighUserModel userInfoModel = (HighUserModel) sessionObject.getObject();


            if (StringUtils.isBlank(object.getString("customerMobile")) ||
                    StringUtils.isBlank(object.getString("regionId")) ||
                    StringUtils.isBlank(object.getString("storeCode")) ||
                    object.getInteger("productType") == null ||
                    object.getInteger("platformId") == null ||
                    object.getJSONArray("orderItems") == null
            ) {
                log.error("addOrder error!");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            SecRegion region = commonService.getParentByRegion(object.getLong("regionId"));
            if (region != null) {
                BsCompany bsCompany = bsCompanyService.selectCompanyByRegion(region.getRegionId().toString());
                object.put("companyId", String.valueOf(bsCompany.getId()));
            }

            object.put("userId", userInfoModel.getHighUser().getId());

            return ResponseMsgUtil.success(highOrderService.insertThirdProduct(object));

        } catch (Exception e) {
            log.error("HighOrderController -> addOrder() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }
}
